/*	InOut.h
*
*	Implementation of classes that handle input and output.
*
*	Written by Cathy Saxton
*	robotics@idleloop.com
*/

#include <avr/io.h>
#include "main.h"

#define msDebounce 5	// milliseconds to wait for button debounce

/*
*	IN -- input class, e.g. for switches
*/

/* constructor for IN class
   - sets up the specified register and pin for input
   - enables/disables the pull-up resistor
   - stores values to member variables
     ~ names start with "m_" to indicate members
     ~ m_bit is the bit corresponding to the pin
	 ~ m_preg is a pointer to the PIN register
*/
IN::IN(char reg, char pin, bool fPullup)
{
	volatile uint8_t *preg;	// pointer to PORT register (for pull-up R)

	m_fBeep = false;
	
	m_bit = 0x01 << pin;
	
	/* set direction for input; set m_preg, preg */
	switch (reg)
	{
	default:
		m_bit = 0;	// error-handling
		// fall through so m_preg gets set...
	case 'b':
	case 'B':
		DDRB &= ~m_bit;
		m_preg = &PINB;
		preg = &PORTB;
		break;
	case 'c':
	case 'C':
		DDRC &= ~m_bit;
		m_preg = &PINC;
		preg = &PORTC;
		break;
	case 'd':
	case 'D':
		DDRD &= ~m_bit;
		m_preg = &PIND;
		preg = &PORTD;
		break;
	}
	
	if (fPullup)
		*preg |= m_bit;		// enable pull-up resistor
	else
		*preg &= ~m_bit;	// disable pull-up resistor
}

/* returns true if pin is reading high, else false */
bool IN::IsHigh() const
{
	return *m_preg & m_bit;
}

/* checks for button press; if not pressed, returns false; else
   waits for release (including debouncing both press & release);
   sounds a beep/buzz when pressed if m_fBeep is true */
bool IN::WasPressed() const
{
	if (IsOpen())
		return false;

	if (m_fBeep)
		g_buzzer.Buzz(toneHigh, 50);

	g_timer.WaitMs(msDebounce);
	while (IsClosed())
		;
	g_timer.WaitMs(msDebounce);
	return true;
}

/*
*	OUT -- output class, e.g. for LEDs
*/

/* constructor for OUT class
   - sets up the specified register and port for output
   - sets the given output value
   - stores values to member variables
     ~ names start with "m_" to indicate members
     ~ m_bit is the bit corresponding to the port
	 ~ m_preg is a pointer to the PORT register
*/
OUT::OUT(char reg, char port, bool fStartHigh)
{
	m_bit = 0x01 << port;
	
	/* set direction for output; set m_preg */
	switch (reg)
	{
	default:	m_bit = 0; // fall through so m_preg gets set...
	case 'b':
	case 'B':	DDRB |= m_bit; m_preg = &PORTB;	break;
	case 'c':
	case 'C':	DDRC |= m_bit; m_preg = &PORTC;	break;
	case 'd':
	case 'D':	DDRD |= m_bit; m_preg = &PORTD;	break;
	}
	
	/* set initial state */
	if (fStartHigh)
		SetHigh();
	else
		SetLow();
}

/* set output high */
void OUT::SetHigh() const
{
	*m_preg |= m_bit;
}

/* set output low */
void OUT::SetLow() const
{
	*m_preg &= ~m_bit;
}

/* toggle output */
void OUT::Toggle() const
{
	*m_preg ^= m_bit;
}

/* set LED on or off */
void OUT::Set(char fOn) const
{
	if (fOn)
		On();
	else
		Off();
}
