/*	LCD.h
*
*	Class for controlling the LCD.
*
*	Assumes the following data connections:
*	LCD		ATmega8
*	---		-------
*	DB7		PD7		[high bit]
*	DB6		PB5
*	DB5		PB4
*	DB4		PB3
*
*	Written by Cathy Saxton
*	robotics@idleloop.com
*/

#pragma once

#include "InOut.h"
#include "Timer.h"

class LCD
{
public:
	/* the constructor needs to know which outputs to use!
	   assumes D0-7 for data */
	LCD(OUT *poutRS, OUT *poutRW, OUT *poutE);

	/* prints given string on the specified line (1 or 2);
	   remainder of (visible) line will be padded with spaces by default */
	void Print(const char *psz, char line, bool fPad = true) const;
	
	/* sets current "write" location to the specified position
	   (ich, 0-based) on the specified line (1 or 2) */
	void SetPos(char ich, char line) const;
	/* writes a character at the current location, updates
	   current location to next space (to right) */
	void WriteCh(char ch) const;
	
	/* clears the display and resets the current location to the
	   upper left */
	void ClearAndReturn() const;
	/* show the cursor (blinking or not) */
	void ShowCursor(bool fBlink) const;
	/* hide the cursor */
	void HideCursor() const;
	/* moves the cursor; fRight indicates direction, cch is
	   count of characters to move [default = 1] */
	void MoveCursor(bool fRight, char cch = 1) const;
	/* shifts the display; fRight indicates direction, cch is
	   count of characters to shift [default = 1] */
	void ShiftDisplay(bool fRight, char cch = 1) const;

private:
	void Init() const;
	void SetMemAddr(unsigned char ch) const;
	void WriteCommand(bool fRS, unsigned char ch) const;
	void WriteNibble(bool fRS, unsigned char ch) const;
	
	OUT *m_poutRS;
	OUT *m_poutRW;
	OUT *m_poutE;

	/* static variables: per class, not instance */
	static bool s_fInit;
};
